#ifndef GUARD_argck_h
#define GUARD_argck_h

// argck.h

// Parameter   Description
// argc        The number of commmand line arguments, including the 
//             command (program name) itself.
// argv        The command line arguments, including the command 
//             (program name) itself. argv is an array of C-style
//             strings.
// arg         Argument that is compared to the elements of argv.
//             arg is a C-style string.
//
// argck determines whether arg matches any command line arguments 
// which are stored in argv. If so, the function returns the number 
// corresponding to the argument. If not, the function returns 0.
//
// It may seem that the function should return bool true if there
// is a match, and bool false otherwise. However, this is not as
// useful as returning the argument number. To understand this claim,
// consider this command line:
//      edit_traces -zero 15 -delete 37 48 <n124.dat >n124.dat
// Program edit_traces sets to zero trace 15 and deletes traces
// 37 through 48. Function argck is called to determine whether
// "-delete" is in the command line; the function returns "3".
// Program edit_traces then knows that arguments 4 and 5 specify the
// range of traces to be deleted.
//
// The original function was given to me by Greg Duckworth.
//

#include <cstring>			// strcmp

int argck( int argc, char** argv, char* arg )

{
	for( int i = 1 ; i < argc ; i++ ) {
		if( !strcmp( argv[i], arg ) )
            return(i) ;					// The two strings match.
	}
	
	return(0) ;
}

#endif
