#ifndef GUARD_seg2_fdb_h
#define GUARD_seg2_fdb_h

// seg2_fdb.h

#include <iostream>					// ostream
#include <fstream>					// ifstream, ofstream
#include <vector>					// vector
#include <string>					// string
#include "seg2_sb.h"
#include "seg2_misc.h"

class File_Descriptor {
	friend class seg2record ;

public:
private:

	typedef std::vector<unsigned long int>::size_type oindex ;

	// Default constructor.
	File_Descriptor() ;

	// Read and check the file descriptor block.
	std::ifstream& read_check( std::ifstream& ) ;

	// Write the file descriptor block to a binary stream.
	std::ofstream& write( std::ofstream& ) ;

	// Write only the offsets to a binary stream.
	std::ofstream& write_offsets( std::ofstream& ) const ;

	// Update an offset in the trace pointer sub-block.
	void update_offset( const oindex, const unsigned long int ) ;

	// Update the file descriptor block when traces are deleted.  
	void delete_traces( const std::vector<unsigned long int>::size_type,
		const std::vector<unsigned long int>::size_type ) ; 

	// Update the file descriptor block when a trace is added.
	void add_trace() ;

	// Set a keyword and its value.
	void set_keyword( const std::string, const std::string ) ;

	// Delete a keyword and its associated value.
	void delete_keyword( const std::string ) ;

	// Check whether the file descriptor blocks of this record and 
	// another record are compatible for various operations like stacking.
	bool compatible( const File_Descriptor& ) const ;

	// Accessor functions.
	char get_sizeof_st() const { return sizeof_st ; }
	char get_fstc() const { return fstc ; }
	char get_sstc() const { return sstc ; }
	unsigned short int get_n_traces() const { return n_traces ; }
	unsigned long int get_offset( unsigned long int i ) const 
	{ return offsets[i] ; }
	std::string get_units() const { return sb.get_value( "UNITS" ) ; }

	// Get a portion of the file descriptor block.
	Record_Info	get_record_info() const ;

	// Print the entire file descriptor block in a text (ascii) file.
	std::ostream& print( std::ostream& ) const ;

	// Print a portion of the file descriptor block in a text (ascii) file. 
	std::ostream& get_record_info( std::ostream& ) const ;

	// Read the file descriptor block from a binary stream.
	std::ifstream& read( std::ifstream& ) ;

	// Check the file descriptor block.
	void check( ) const ;

	static const short int required_block_id = 0x3a55 ;
										// Required value of the block id.
	short int block_id ;				// file descriptor block id 
	short int revision_num ;			// revision number 
	unsigned short int sizeof_tpsb ;	// size of trace pointer sub-block (in bytes)
	unsigned short int n_traces ;		// number of traces in this file 
	char sizeof_st ;					// size of string terminator (either 1 or 2) 
	char fstc ;							// first string terminator character 
	char sstc ;							// second string terminator character 
	char sizeof_lt ;					// size of line terminator (either 1 or 2) 
	char fltc ;							// first line terminator character 
	char sltc ;							// second line terminator character 
	char reserved[18] ;
	std::vector<unsigned long int> offsets ; // "offsets" are called "pointers"
										// in the article. Each offset is the number
										// of bytes between the start of the file and
										// the corresponding trace descriptor block.
	String_Block sb ;					// String block.


} ;

// Check whether the keyword is a standard keyword for the
// file descriptor block.
bool check_fdb_keyword( const std::string ) ;

#endif