// This file contains functions that are used by both 
// seg2_fdb and seg2_tb.

#include "seg2_misc.h"

// In the SEG-2 format, all blocks must start on a 32-bit boundary.
// To understand this, let's define the "offset" as the number of
// bytes between the beginning of the file and the start of a block.
// In the SEG-2 format, the offset must be an integer multiple of
// 4 bytes. (Recall that 32 bits equals 4 bytes.) 
//
// Thus, this function checks whether the file has reached a 32-bit 
// boundary. If not, the function adds enough bits to reach the
// boundary. 

std::ofstream& adjust_file_boundary( std::ofstream& os ) 
{

	// A 32 bits corresponds to 4 bytes. Therefore, to determine
	// whether the file is at a boundary, divide the file size (which
	// is specified in bytes) by 4; the following table indicates
	// the number of bytes needed to put the file at a boundary.
	//
	// Remainder	Number of bytes
	// 0			0 (The file is at a boundary.)
	// 1			3
	// 2			2
	// 3			1

	long int remainder = os.tellp( ) % 4 ;

	long int needed_bytes ;
	switch( remainder ) {
		case 0 : 
			needed_bytes = 0 ;
			break ;
		case 1 :
			needed_bytes = 3 ;
			break ;
		case 2 :
			needed_bytes = 2 ;
			break ;
		case 3 :
			needed_bytes = 1 ;
			break ;
	}

	char c = ' ' ;
	for( int i = 0 ; i != needed_bytes ; ++i )
		os.write( &c, sizeof(c) ) ;

	return os ;
}
