#include <algorithm>				// min
#include <utility>					// make_pair
#include "seg2_sb.h"
#include "D:\code\c++\my lib\str_manip.h"
#include "D:\code\c++\my lib\Read_String.h"

using std::string ;
using std::cerr ;
using std::endl ;
using std::min ;
using std::make_pair ;
using std::map ;

// Default constructor.
String_Block::String_Block()
{
	sb.clear() ;
}

// Read the string block from a binary stream.
std::ifstream& String_Block::read( std::ifstream& is, const char sizeof_st )
{

	// remove previous contents, if any
	sb.clear() ;

	string::size_type length_st = sizeof_st ;

	unsigned short int length ;

	is.read( (char*) &length, sizeof(length) ) ;

	string field ;

	while( length != 0 ) {

		Read_String( is, field, length-sizeof(length) ) ;

		// Split field into keyword and value.
		// Exclude from value the string termination characters;
		// Also trim the beginning and the end of value.
		string::size_type i = 
			min( field.find_first_of( " " ), field.find_first_of( "\t" ) ) ;
		string keyword = field.substr( 0, i ) ;
		string::size_type n = field.size() - length_st - ( i+1 ) ;
		string value = str_trim( field.substr( i+1, n ), "b" ) ;

		// Insert the keyword and value into sb.
		sb.insert( make_pair( keyword, value) ) ;

		is.read( (char*) &length, sizeof(length) ) ;
	}

	return is ;
}

// Write the string block to a binary stream.
std::ofstream& String_Block::write
( std::ofstream& os, const char sizeof_st, const char fstc, const char sstc ) const
{

	map<string,string>::const_iterator it_note ;
	bool note_present = false ;
	
	string str_end ;
	if( static_cast<int>( sizeof_st ) == 1 )
		str_end = fstc ;
	else
		str_end = fstc + sstc ;

	for( map<string,string>::const_iterator it = sb.begin() ;
		it != sb.end() ; ++it ) 
	{

		if( it->first == "NOTE" ) {
			// If the current keyword is "NOTE", don't write it now.
			it_note = it ;
			note_present = true ;
			continue ;
		}

		string field = it->first + " " + it->second + str_end ;

		unsigned short int length = sizeof(unsigned short int) +
			static_cast<unsigned short int>( field.size() );

		os.write( (char*) &length, sizeof(length) ) ;
		os.write( field.data(), std::streamsize(field.size()) ) ;

	}

	// If there is a "NOTE", write it now.
	if( note_present ) {

		string field = it_note->first + " " + it_note->second + str_end ;

		unsigned short int length = sizeof(unsigned short int) +
			static_cast<unsigned short int>( field.size() ) ;

		os.write( (char*) &length, sizeof(length) ) ;
		os.write( field.data(), std::streamsize(field.size()) ) ;

	}

	// Write a '0' to indicate the end of the string sub-block.

	unsigned short int j = 0 ;
	os.write( (char*) &j, sizeof(j) ) ;

	return os ;
}

// Print the entire string block in a text (ascii) file.
std::ostream& String_Block::print( std::ostream& os ) const
{

	map<string,string>::const_iterator it_note ;
	bool note_present = false ;
	
	for( map<string,string>::const_iterator it = sb.begin() ;
		it != sb.end() ; ++it ) 
	{

		if( it->first == "NOTE" ) {
			// If the current keyword is "NOTE", don't write it now.
			it_note = it ;
			note_present = true ;
			continue ;
		}

		os << it->first << ":  " << it->second << endl ;

	}

	// If there is a "NOTE", write it now.
	if( note_present ) 
		os << it_note->first << ":  " << it_note->second << endl ;

	return os ;
}


// Set a keyword and its value.
void String_Block::set_keyword( const string keyword, const string value ) 
{

	// If the keyword is already in sb, then remove the keyword
	// and its value. 
	map<string,string>::iterator it = sb.find( keyword ) ;
	if( it != sb.end() ) 
		sb.erase(it) ;

	// Insert the keyword and the value into sb.
	sb.insert( make_pair( keyword, value ) ) ;

}

// Delete a keyword and its associated value.
void String_Block::delete_keyword( const string keyword ) 
{

	map<string,string>::iterator it = sb.find( keyword ) ;
	if( it != sb.end() ) 
		sb.erase(it) ;
	else
		cerr << "Class String_Block, Function delete_keyword." << endl
			<< "Keyword " << keyword << " not found in string block." << endl ;

}

// Clear all keywords and values. 
void String_Block::clear()
{
	sb.clear() ;
}

// Get the value associated with a keyword.
string String_Block::get_value( const string keyword ) const 
{
	string value ;

	map<string,string>::const_iterator it = sb.find( keyword ) ;

	if( it != sb.end() )
		value = it->second ;
	else {
		cerr << "Class: String_Block   Function: get_value   " <<
			"Error: Keyword " << keyword << " not found." << endl ;
		value = "" ;
	}		

	return  value ;
}

