#ifndef GUARD_seg2record_h
#define GUARD_seg2record_h

// seg2record.h

#include <iostream>					// ifstream, ofstream, ostream
#include <vector>					// vector
#include <string>					// string

#include "seg2_fdb.h"
#include "seg2_tb.h"
#include "seg2_misc.h"

// Class seg2record serves as an interface to class File_Descriptor
// and class Trace_Block.

class seg2record {

public:

	typedef std::vector<Trace_Block>::size_type vindex ;

	// Default constructor.
	seg2record() ;

	// Constructor that initializes the object by reading a 
	// record from a binary file.
	seg2record( std::ifstream& ) ;

	// Read a record from a binary stream.
	std::ifstream& read( std::ifstream& ) ;

	// Write a record to a binary stream.
	std::ofstream& write( std::ofstream& ) ;

	// Zero one or more traces.
	void zero_traces( const vindex, const vindex ) ;

	// Scale the traces in a record.
	seg2record& operator*=( const double ) ;

	// Stack (+) the traces in this and another record.
	seg2record& operator+=( const seg2record& ) ;

	// Stack (-) the traces in this and another record.
	seg2record& operator-=( seg2record& ) ;

	// Delete one or more traces.
	void delete_traces( const vindex, const vindex ) ;

	// Set a keyword in the file descriptor block.
	void set_keyword( const std::string, const std::string ) ;

	// Set a keyword in the trace descriptor blocks (of a Trace_Block).
	void set_keyword( const vindex, const vindex, 
		const std::string, const std::string ) ;

	// Delete a keyword from the file descriptor block.
	void delete_keyword( const std::string ) ;

	// Delete a keyword from the trace descriptor blocks (of a Trace_Block).
	void delete_keyword( const vindex, const vindex, const std::string ) ;

	// Create a SEG-2 trace and append it to the record.
	void create_trace( const Trace_Info, const float* ) ;

	// Get selected information about the record (file descriptor block).
	Record_Info	get_record_info() const ;

	// Print selected information about the record (file descriptor block)
	// in an text (ascii) file.
	std::ostream& get_record_info( std::ostream& ) const ;

	// Get selected information about the traces.
	std::vector<Trace_Info> get_trace_info() const ;

	// Print almost the entire the record in a text (ascii) file.
	// (The traces themselves are not printed --- only their rms
	// values are.)
	std::ostream& print( std::ostream& ) const ;

	// Print selected information about the traces in an text (ascii) file. 
	std::ostream& get_trace_info( std::ostream& os ) const ;

	// Get the traces.
	void get_traces( float* ) const ;

	// Get an entire Trace_Block.
	Trace_Block get_trace_block ( const vindex ) const ;

	// Append a Trace_Block to the record.
	void append_trace_block( const Trace_Block ) ;

private:
	// Check whether this record and another record are compatible for
	// various operations like stacking.
	bool compatible( const seg2record& ) const ;

	File_Descriptor fdb ;
	std::vector<Trace_Block> tb ;
} ;

#endif