#include <cctype>					// isspace
#include <algorithm>				// find_if
#include "str_manip.h"

using std::string ;
using std::vector ;
using std::find_if ;

// 
// Name:
//		str_trim
//
// Purpose:
//		This function removes non-standard characters from either the
//		beginning of a string, the end of a string, or both. 
//		Non-standard characters refer to those ASCII characters
//		whose decimal equivalents are either less than 33 or equal
//		to 127. Examples of non-standard characters are the tab
//		(no. 9), the space (no. 32), and the delete (no. 127).
//
// Author:
//		Karl J. Ellefsen
//
// Usage:
//		Argument	Meaning
//		str			The string from which the non-standard characters
//					will be removed.
//		which		A string indicating where the characters may be
//					removed. The possible values for string which are
//					"f" for the front of the string, "e" for
//					the end of the string, and "b" for both
//					the front and the end.
//
//		The function returns the editted string.
//
// Required files:
//		none
//
//	Additional comments:
//		The code conforms to ANSI Standard C++.
//
// Modification History:
//		2002.11		Written and tested.
//


string str_trim( const string& str, const string which )
{
	string s = str ;

	// trim the beginning of the string
	if( which == "f" || which == "b" ) {

		// At the end of this loop, i is the index of first character
		// that will be kept. In other words, all characters whose
		// indicies are smaller than i are erased.
		size_t i ;
		for( i = 0 ; i != s.size() ; ++i ) {
			int c = static_cast<int>( s[i] ) ;
			if( c <= 32 || c == 127 )
				continue ;
			else
				break ;
		}

		if( i > 0 )
			s.erase( s.begin(), s.begin()+i ) ;
	}

	// trim the end of the string
	if( which == "e" || which == "b" ) {

		// At the end of this loop, i is the index of first character
		// that will be kept. In other words, all characters whose
		// indicies are greater than i are erased.
		size_t i ;
		for( i = s.size()-1 ; i != -1 ; --i ) {
			int c = static_cast<int>( s[i] ) ;
			if( c <= 32 || c == 127 )
				continue ;
			else
				break ;
		}

		if( i < s.size()-1 )
			s.erase( s.begin()+i, s.end() ) ;
	}

	return s ;
}

// Function str_split separates a string into sub-strings. The criterion
// for making a separation is a blank space. Each sub-string is one
// element in a vector (of strings), and this vector is returned.
//
// This function was originally published in 
// Koenig, A. and Moo, B. E., 2000, Accelerated C++ --- Practical
// Program by Example: New York, Addison-Wesley, 336 p.
//
// The function is available on the internet at
// http://www.acceleratedcpp.com/

bool space( char c )
{
	return static_cast<bool>( isspace(c) ) ;
}

bool not_space( char c )
{
	return !( static_cast<bool>( isspace(c) ) ) ;
}

vector<string> str_split( const string& str )
{
	typedef string::const_iterator iter ;
	vector<string> ret ;

	iter i = str.begin() ;

	while( i != str.end() ) {

		i = find_if( i, str.end(), not_space ) ;

		iter j = find_if( i, str.end(), space ) ;

		if( i != str.end() ) 
			ret.push_back( string(i,j) ) ;
		i = j ;
		
	}
	return ret ;
}